searchData={"items":[{"type":"module","title":"ECSx","doc":"ECSx is an Entity-Component-System (ECS) framework for Elixir.\n\nIn ECS:\n\n* Every game object is an Entity, represented by a unique ID.\n* The data which comprises an Entity is split among many Components.\n* Game logic is split into Systems, which update the Components every server tick.\n\nUnder the hood, ECSx uses Erlang Term Storage (ETS) to store active Components in memory.\nA single GenServer manages the ETS tables to ensure strict serializability and customize\nthe run order for Systems.","ref":"ECSx.html"},{"type":"module","title":"Configuration - ECSx","doc":"You may configure various settings for ECSx in the application environment\n(usually defined in `config/config.exs`):\n\n    config :ecsx,\n      manager: MyApp.Manager,\n      tick_rate: 20,\n      persist_interval: :timer.seconds(15),\n      persistence_adapter: ECSx.Persistence.FileAdapter,\n      persistence_file_location: \"components.persistence\"\n\n  * `:manager` - This setting defines the module and path for your app's ECSx Manager.\n    When only a module name is given here, the path will be inferred using the standard\n    directory conventions (e.g. `MyApp.Manager` becomes `lib/my_app/manager.ex`).\n    If you are using a different structure for your directories, you can instead use a tuple\n    including the `:path` option (e.g. `{ManagerModule, path: \"lib/path/to/file.ex\"}`)\n  * `:tick_rate` - This controls how many times per second each system will run.  Setting a higher\n    value here can make a smoother experience for users of your app, but will come at the cost\n    of increased server load.  Increasing this value beyond your hardware's capabilities will\n    result in instability across the entire application, worsening over time until eventually\n    the application crashes.\n  * `:persist_interval` - ECSx makes regular backups of all components marked for persistence.\n    This setting defines the length of time between each backup.\n  * `:persistence_adapter` - If you have a custom adapter which implements\n    `ECSx.Persistence.Behaviour`, you can set it here to replace the default `FileAdapter`.\n  * `:persistence_file_location` - If you are using the default `FileAdapter` for persistence,\n    this setting allows you to define the path for the backup file.","ref":"ECSx.html#module-configuration"},{"type":"function","title":"ECSx.manager/0","doc":"Returns the ECSx manager module.\n\nThis is set in your app configuration:\n\n```elixir\nconfig :ecsx, manager: MyApp.Manager\n```","ref":"ECSx.html#manager/0"},{"type":"function","title":"ECSx.manager_path/0","doc":"Returns the path to the ECSx manager file.\n\nThis is inferred by your module name.  If you want to rename or move the\nmanager file so the path and module name are no longer in alignment, use\na custom `:path` opt along with the manager module, wrapped in a tuple.","ref":"ECSx.html#manager_path/0"},{"type":"function","title":"Examples - ECSx.manager_path/0","doc":"```elixir\n# standard path: lib/my_app/manager.ex\nconfig :ecsx, manager: MyApp.Manager\n\n# custom path: lib/foo/bar/baz.ex\nconfig :ecsx, manager: {MyApp.Manager, path: \"lib/foo/bar/baz.ex\"}\n```","ref":"ECSx.html#manager_path/0-examples"},{"type":"function","title":"ECSx.persist_interval/0","doc":"Returns the frequency of component persistence.\n\nThis defaults to 15 seconds, and can be changed in your app configuration:\n\n```elixir\nconfig :ecsx, persist_interval: :timer.minutes(1)\n```","ref":"ECSx.html#persist_interval/0"},{"type":"function","title":"ECSx.tick_rate/0","doc":"Returns the tick rate of the ECSx application.\n\nThis defaults to 20, and can be changed in your app configuration:\n\n```elixir\nconfig :ecsx, tick_rate: 15\n```","ref":"ECSx.html#tick_rate/0"},{"type":"module","title":"ECSx.ClientEvents","doc":"A store to which clients can write, for communication with the ECSx backend.\n\nEvents are created from the client process by calling `add/2`, then retrieved by the handler\nsystem using `get_and_clear/0`.  You will be required to create the handler system yourself -\nsee the [tutorial project](web_frontend_liveview.html#handling-client-events) for a detailed example.","ref":"ECSx.ClientEvents.html"},{"type":"function","title":"ECSx.ClientEvents.add/2","doc":"Add a new client event.\n\nThe first argument is the entity which spawned the event.\nThe second argument can be any representation of the event, usually either an atom or a tuple\ncontaining an atom name along with additional metadata.","ref":"ECSx.ClientEvents.html#add/2"},{"type":"function","title":"Examples - ECSx.ClientEvents.add/2","doc":"# Simple event requiring no metadata\n    ECSx.ClientEvents.add(player_id, :spawn_player)\n\n    # Event with metadata\n    ECSx.ClientEvents.add(player_id, {:send_message_to, recipient_id, message})","ref":"ECSx.ClientEvents.html#add/2-examples"},{"type":"function","title":"ECSx.ClientEvents.get_and_clear/0","doc":"Returns the list of events, simultaneously clearing it.\n\nThis function guarantees that each event is returned exactly once.","ref":"ECSx.ClientEvents.html#get_and_clear/0"},{"type":"type","title":"ECSx.ClientEvents.id/0","doc":"","ref":"ECSx.ClientEvents.html#t:id/0"},{"type":"behaviour","title":"ECSx.Component","doc":"A Component labels an entity as having a certain attribute, and holds any data needed to model that attribute.\n\nFor example, if Entities in your application should have a \"color\" value, you will create a Component type called `Color`.  This allows you to add a color component to an Entity with `add/2`, look up the color value for a given Entity with `get_one/1`, get all Entities' color values with `get_all/1`, remove the color value from an Entity altogether with `remove/1`, or test whether an entity has a color with `exists?/1`.\n\nUnder the hood, we use ETS to store the Components in memory for quick retrieval via Entity ID.","ref":"ECSx.Component.html"},{"type":"behaviour","title":"Usage - ECSx.Component","doc":"Each Component type should have its own module, where it can be optionally configured.\n\n    defmodule MyApp.Components.Color do\n      use ECSx.Component,\n        value: :binary\n    end","ref":"ECSx.Component.html#module-usage"},{"type":"behaviour","title":"Options - ECSx.Component","doc":"* `:value` - The type of value which will be stored in this component type.  Valid types are: `:atom, :binary, :datetime, :float, :integer`\n  * `:index` - When `true`, the `search/1` function will be much more efficient, at the cost of slightly higher write times.  Defaults to `false`\n  * `:log_edits` - When `true`, log messages will be emitted for each component added, updated, or removed.  Defaults to `false`\n  * `:read_concurrency` - When `true`, enables read concurrency for this component table.  Only set this if you know what you're doing.  Defaults to `false`","ref":"ECSx.Component.html#module-options"},{"type":"callback","title":"ECSx.Component.add/3","doc":"Creates a new component.","ref":"ECSx.Component.html#c:add/3"},{"type":"callback","title":"Options - ECSx.Component.add/3","doc":"* `:persist` - When `true`, this component will persist across app reboots.  Defaults to `false`","ref":"ECSx.Component.html#c:add/3-options"},{"type":"callback","title":"Example - ECSx.Component.add/3","doc":"# Add an ArmorRating component to entity `123` with value `10`\n    # If the app shuts down, this component will be removed\n    ArmorRating.add(123, 10)\n\n    # This ArmorRating component will be persisted after app shutdown,\n    # and automatically re-added to entity `123` upon next startup\n    ArmorRating.add(123, 10, persist: true)","ref":"ECSx.Component.html#c:add/3-example"},{"type":"callback","title":"ECSx.Component.at_least/1","doc":"Look up all components where the value is greater than or equal to `min`.\n\nThis function only works for numerical component types (`:value` set to either\n`:integer` or `:float`). Other value types will raise `UndefinedFunctionError`.","ref":"ECSx.Component.html#c:at_least/1"},{"type":"callback","title":"Example - ECSx.Component.at_least/1","doc":"# Get all PlayerExperience components where value >= 2500\n    PlayerExperience.at_least(2500)","ref":"ECSx.Component.html#c:at_least/1-example"},{"type":"callback","title":"ECSx.Component.at_most/1","doc":"Look up all components where the value is less than or equal to `max`.\n\nThis function only works for numerical component types (`:value` set to either\n`:integer` or `:float`). Other value types will raise `UndefinedFunctionError`.","ref":"ECSx.Component.html#c:at_most/1"},{"type":"callback","title":"Example - ECSx.Component.at_most/1","doc":"# Get all PlayerHealth components where value <= 10\n    PlayerHealth.at_most(10)","ref":"ECSx.Component.html#c:at_most/1-example"},{"type":"callback","title":"ECSx.Component.between/2","doc":"Look up all components where the value is greater than or equal to `min` and less\nthan or equal to `max`.\n\nThis function only works for numerical component types (`:value` set to either\n`:integer` or `:float`). Other value types will raise `UndefinedFunctionError`.","ref":"ECSx.Component.html#c:between/2"},{"type":"callback","title":"Example - ECSx.Component.between/2","doc":"# Get all RespawnCount components where 51 <= value <= 100\n    RespawnCount.between(51, 100)","ref":"ECSx.Component.html#c:between/2-example"},{"type":"callback","title":"ECSx.Component.exists?/1","doc":"Checks if an entity has a component of this type.","ref":"ECSx.Component.html#c:exists?/1"},{"type":"callback","title":"ECSx.Component.get/2","doc":"Look up a component and return its value.\n\nIf a `default` value is provided, that value will be returned if no result is found.\n\nIf `default` is not provided, this function will raise an `ECSx.NoResultsError` if no result is found.","ref":"ECSx.Component.html#c:get/2"},{"type":"callback","title":"Example - ECSx.Component.get/2","doc":"# Get the Velocity for entity `123`, which is known to already exist\n    Velocity.get(123)\n\n    # Get the Velocity for entity `123` if it exists, otherwise return `nil`\n    Velocity.get(123, nil)","ref":"ECSx.Component.html#c:get/2-example"},{"type":"callback","title":"ECSx.Component.get_all/0","doc":"Look up all components of this type.","ref":"ECSx.Component.html#c:get_all/0"},{"type":"callback","title":"Example - ECSx.Component.get_all/0","doc":"# Get all velocity components\n    Velocity.get_all()","ref":"ECSx.Component.html#c:get_all/0-example"},{"type":"type","title":"ECSx.Component.id/0","doc":"","ref":"ECSx.Component.html#t:id/0"},{"type":"callback","title":"ECSx.Component.remove/1","doc":"Removes this component type from an entity.","ref":"ECSx.Component.html#c:remove/1"},{"type":"callback","title":"ECSx.Component.search/1","doc":"Look up all IDs for entities which have a component of this type with a given value.\n\nThis function is significantly optimized by the `:index` option.  For component\ntypes which are regularly searched, it is highly recommended to set this option to `true`.","ref":"ECSx.Component.html#c:search/1"},{"type":"callback","title":"Example - ECSx.Component.search/1","doc":"# Get all entities with a velocity of `60`\n    Velocity.search(60)","ref":"ECSx.Component.html#c:search/1-example"},{"type":"callback","title":"ECSx.Component.update/2","doc":"Updates an existing component's value.\n\nThe component's `:persist` option will remain unchanged. (see `add/3`)","ref":"ECSx.Component.html#c:update/2"},{"type":"callback","title":"Example - ECSx.Component.update/2","doc":"ArmorRating.add(123, 10)\n    # Increase the ArmorRating value from `10` to `15`\n    ArmorRating.update(123, 15)","ref":"ECSx.Component.html#c:update/2-example"},{"type":"type","title":"ECSx.Component.value/0","doc":"","ref":"ECSx.Component.html#t:value/0"},{"type":"behaviour","title":"ECSx.Manager","doc":"The Manager for your ECSx application.\n\nIn an ECSx application, the Manager is responsible for:\n\n  * starting up ETS tables for each Component Type, where the Components will be stored\n  * prepopulating the game content into memory\n  * keeping track of the Systems to run, and their run order\n  * running the Systems every tick\n\n## `components/0` and `systems/0`\n\nYour Manager module must contain two zero-arity functions called `components` and `systems`\nwhich return a list of all Component Types and Systems in your application.  The order of\nthe Component Types list is irrelevant, but the order of the Systems list is very important,\nbecause the Systems are run consecutively in the given order.\n\n## `setup/0` and `startup/0`\n\nManager modules may also implement two optional functions for loading all the necessary\ncomponent data for your app before any Systems run or users connect.\n\nThe `setup/0` function runs only *once*, when you start your app for the first time, while\nthe `startup/0` function runs *every* time the app starts, including the first\n(after `setup/0` is run). The Manager uses the Persistence layer to determine if this\nis a fresh server or a subsequent start.\n\nThese functions will be run during the Manager's initialization. The Component tables\nwill be created before they are executed.","ref":"ECSx.Manager.html"},{"type":"behaviour","title":"Example - ECSx.Manager","doc":"```\ndefmodule YourApp.Manager do\n  use ECSx.Manager\n\n  def setup do\n    for tree <- YourApp.Map.trees() do\n      YourApp.Components.XPosition.add(tree.id, tree.x_coord, persist: true)\n      YourApp.Components.YPosition.add(tree.id, tree.y_coord, persist: true)\n      YourApp.Components.Type.add(tree.id, \"Tree\", persist: true)\n    end\n  end\n\n  def startup do\n    for spawn_location <- YourApp.spawn_locations() do\n      YourApp.Components.SpawnLocation.add(spawn_location.id)\n      YourApp.Components.Type.add(spawn_location.id, spawn_location.type)\n      YourApp.Components.XPosition.add(spawn_location.id, spawn_location.x_coord)\n      YourApp.Components.YPosition.add(spawn_location.id, spawn_location.y_coord)\n    end\n  end\nend\n```","ref":"ECSx.Manager.html#module-example"},{"type":"callback","title":"ECSx.Manager.setup/0","doc":"Loads component data for first app launch.\n\nThis will run only once, the first time you start your app.  It runs after component tables\nhave been initialized, before any systems have started.\n\nExcept for very rare circumstances, all components added here should have `persist: true`","ref":"ECSx.Manager.html#c:setup/0"},{"type":"callback","title":"ECSx.Manager.startup/0","doc":"Loads ephemeral component data each time the app is started.\n\nThis will run on your app's first start (after `setup/0`) and then again during all subsequent\napp reboots.  It runs after component tables have been initialized, before any systems have started.\n\nExcept for very rare circumstances, components added here should *not* be persisted.","ref":"ECSx.Manager.html#c:startup/0"},{"type":"behaviour","title":"ECSx.Persistence.Behaviour","doc":"By default, ECSx persists component data by writing a binary file to disk, then reading the file\nwhen the server restarts.  If you would like to use a different method, you can create a module\nwhich implements this behaviour, and update the ECSx configuration to use your module instead of\nthe default.\n\n## `persist_components/2` and `retrieve_components/1`\n\nTo create your own persistence adapter, you only need to implement two functions:\n\n  * `persist_components/2` - This function takes a map, where keys are component type modules, and\n    values are lists of persistable components of that type.  A keyword list of options is also be\n    passed as a second argument.  The function should store the data, then return `:ok`.\n  * `retrieve_components/1` - This function takes a list of options, and should return\n    `{:ok, component_map}` where `component_map` stores lists of component tuples as values,\n    with the keys being the component type module corresponding to each list.","ref":"ECSx.Persistence.Behaviour.html"},{"type":"behaviour","title":"Configuring ECSx to use a custom persistence adapter - ECSx.Persistence.Behaviour","doc":"Once you have created a persistence adapter module, simply update your application config to use it:\n\n    config :ecsx,\n      ...\n      persistence_adapter: MyAdapterModule","ref":"ECSx.Persistence.Behaviour.html#module-configuring-ecsx-to-use-a-custom-persistence-adapter"},{"type":"type","title":"ECSx.Persistence.Behaviour.components/0","doc":"","ref":"ECSx.Persistence.Behaviour.html#t:components/0"},{"type":"callback","title":"ECSx.Persistence.Behaviour.persist_components/2","doc":"","ref":"ECSx.Persistence.Behaviour.html#c:persist_components/2"},{"type":"callback","title":"ECSx.Persistence.Behaviour.retrieve_components/1","doc":"","ref":"ECSx.Persistence.Behaviour.html#c:retrieve_components/1"},{"type":"behaviour","title":"ECSx.System","doc":"A fragment of game logic which reads and updates Components.\n\nEach System must implement a `run/0` function, which will be called once per game tick.\n\n    defmodule MyApp.FooSystem do\n      @behaviour ECSx.System\n\n      @impl ECSx.System\n      def run do\n        # System logic\n        :ok\n      end\n    end","ref":"ECSx.System.html"},{"type":"callback","title":"ECSx.System.run/0","doc":"Invoked to run System logic.\n\nThis function will be called every game tick.\n\nNote:  A crash inside this function will restart the entire app!","ref":"ECSx.System.html#c:run/0"},{"type":"behaviour","title":"ECSx.Tag","doc":"A component type which does not require a value.  This is useful when the mere presence or absence of a component is all the information we need.\n\nFor example, if we want a component type to model a boolean attribute, such as whether or not players may target a particular entity, we'll use a Tag:\n\n    defmodule MyApp.Components.Targetable do\n      use ECSx.Tag\n    end\n\nThen we can check for targetability with `...Targetable.exists?(entity)` or get a list of all targetable entities with `...Targetable.get_all()`.","ref":"ECSx.Tag.html"},{"type":"behaviour","title":"Options - ECSx.Tag","doc":"* `:read_concurrency` - when `true`, enables read concurrency for this component table.  Only set this if you know what you're doing.  Defaults to `false`","ref":"ECSx.Tag.html#module-options"},{"type":"callback","title":"ECSx.Tag.add/1","doc":"Creates a new tag for a given entity.","ref":"ECSx.Tag.html#c:add/1"},{"type":"callback","title":"ECSx.Tag.exists?/1","doc":"Checks if an entity has this tag.","ref":"ECSx.Tag.html#c:exists?/1"},{"type":"callback","title":"ECSx.Tag.get_all/0","doc":"Gets a list of all entities with this tag.","ref":"ECSx.Tag.html#c:get_all/0"},{"type":"type","title":"ECSx.Tag.id/0","doc":"","ref":"ECSx.Tag.html#t:id/0"},{"type":"callback","title":"ECSx.Tag.remove/1","doc":"Removes this component from an entity.","ref":"ECSx.Tag.html#c:remove/1"},{"type":"task","title":"mix ecsx.gen.component","doc":"Generates a new Component type for an ECSx application.\n\n    $ mix ecsx.gen.component Height integer\n\nThe first argument is the name of the component, followed by the data type of the value.\n\nValid types for the component's value are:\n\n  * atom\n  * binary\n  * datetime\n  * float\n  * integer\n\nIf you know you want components of this type to be indexed for improved `ECSx.Component.search/1` performance,\nyou may include the `--index` option:\n\n    $ mix ecsx.gen.component Name binary --index","ref":"Mix.Tasks.Ecsx.Gen.Component.html"},{"type":"task","title":"mix ecsx.gen.system","doc":"Generates a new System for an ECSx application.\n\n    $ mix ecsx.gen.system Foo\n\nThe only argument accepted is a module name for the System.","ref":"Mix.Tasks.Ecsx.Gen.System.html"},{"type":"task","title":"mix ecsx.gen.tag","doc":"Generates a new ECSx Tag - a Component type which doesn't store any value.\n\n    $ mix ecsx.gen.tag Attackable\n\nThe single argument is the name of the component.","ref":"Mix.Tasks.Ecsx.Gen.Tag.html"},{"type":"task","title":"mix ecsx.setup","doc":"Generates the Manager process which runs an ECSx application.\n\n    $ mix ecsx.setup\n\nThis setup will generate `manager.ex` and empty folders for components and systems.\n\nIf you don't want to generate the folders, you can provide option `--no-folders`","ref":"Mix.Tasks.Ecsx.Setup.html"},{"type":"exception","title":"ECSx.AlreadyExistsError","doc":"","ref":"ECSx.AlreadyExistsError.html"},{"type":"exception","title":"ECSx.MultipleResultsError","doc":"","ref":"ECSx.MultipleResultsError.html"},{"type":"exception","title":"ECSx.NoResultsError","doc":"","ref":"ECSx.NoResultsError.html"},{"type":"extras","title":"Installation","doc":"# Installation\n\nTo create an ECSx application, there are a few simple steps:\n\n  * install Elixir + erlang/OTP\n  * install Phoenix (optional)\n  * create an Elixir/Phoenix project\n  * fetch ECSx as a dependency for your project\n  * run the ECSx setup","ref":"installation.html"},{"type":"extras","title":"Elixir and erlang/OTP - Installation","doc":"If you don't yet have Elixir and erlang/OTP installed on your machine, follow the instructions on the official [Installation Page](https://elixir-lang.org/install.html).","ref":"installation.html#elixir-and-erlang-otp"},{"type":"extras","title":"Phoenix - Installation","doc":"If you plan on hosting your application online, you'll probably want to use Phoenix.  You can skip this step if you only want to run the app locally.  Otherwise, follow the instructions for Phoenix [installation](https://hexdocs.pm/phoenix/installation.html) (the tutorial project will assume you are using Phoenix).","ref":"installation.html#phoenix"},{"type":"extras","title":"Create project - Installation","doc":"If you are using Phoenix, you'll create your new application with the command\n\n```console\n$ mix phx.new my_app\n```\n\nOr for a regular Elixir application (with supervision tree):\n\n```console\n$ mix new my_app --sup\n```","ref":"installation.html#create-project"},{"type":"extras","title":"Install ECSx - Installation","doc":"To use the ECSx framework in your application, it should be added to the list of dependencies in `my_app/mix.exs`:\n\n```\ndefp deps do\n  [\n    {:ecsx, \"~> 0.5\"}\n  ]\nend\n```\n\nThen (from the root directory of your application) run:\n\n```console\n$ mix deps.get\n```","ref":"installation.html#install-ecsx"},{"type":"extras","title":"Setup ECSx - Installation","doc":"With ECSx installed, you can run the setup generator:\n\n```console\n$ mix ecsx.setup\n```\n\nwhich will create the Manager, and two folders to get your project started.  You should now have everything you need to start building!","ref":"installation.html#setup-ecsx"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.5.2 (2025-01-25) - Changelog","doc":"* Fixed an issue where Components with `index: true` were not fully removed with `Component.remove/1`","ref":"changelog.html#v0-5-2-2025-01-25"},{"type":"extras","title":"v0.5.1 (2024-01-24) - Changelog","doc":"* Fixed an issue where some Component or System naming patterns could prevent generators from working properly\n  * Replace deprecated function which was causing compiler warnings (h/t @hl)","ref":"changelog.html#v0-5-1-2024-01-24"},{"type":"extras","title":"v0.5 (2023-09-23) - Changelog","doc":"* Non-unique Component types are no longer allowed (see the [upgrade guide](upgrade_guide.html))\n  * Component modules now accept option `:index` to index components for better `search/2` performance\n  * `mix ecsx.gen.component` now accepts option `--index` to automatically set `index: true`\n  * Component callback `get_one/2` has been renamed `get/2`\n  * Systems' `run/0` no longer requires `:ok` return value\n  * Manager `setup/0` and `startup/0` no longer require `:ok` return value","ref":"changelog.html#v0-5-2023-09-23"},{"type":"extras","title":"v0.4 (2023-06-02) - Changelog","doc":"* Adding ECSx.ClientEvents to your supervision tree is no longer required  \n  * Adding the manager to your supervision tree is no longer required  \n  * Running a generator before ecsx.setup will now raise an error   \n  * Added telemetry events  \n  * Added component persistence, by default saving a binary file to disk  \n  * Persistence file is loaded on app startup  \n  * The interval between saves can be set via application config  \n  * Tick rate is now set in application config  \n  * Manager module (and optional custom path) are now defined in application config  \n  * Added functions `tick_rate/0`, `manager/0`, `persist_interval/0`, and `manager_path/0` to the `ECSx` module for reading the configured values at runtime  \n  * Added callback `add/3` for components and tags, which accepts `persist: true` option, marking the component/tag for persistence across app reboots  \n  * `get_one/1` now raises an error if no results are found  \n  * Added `Component` callback `get_one/2` which accepts a default value to return if no results are found  \n  * `add/{2,3}` now raises if `unique: true` and the component already exists  \n  * Added `Component` callback `update/2` for updating an existing component's value, while maintaining the previously set `:persist` option  \n  * Manager `setup` macro is now an optional callback `setup/0` which only runs once, at the server's first startup  \n  * Added a new Manager callback `startup/0` which runs every time the server starts  \n  * Added `Component` callbacks `between/2`, `at_least/1`, and `at_most/1` (only available for integer and float component types)","ref":"changelog.html#v0-4-2023-06-02"},{"type":"extras","title":"v0.3.1 (2023-01-12) - Changelog","doc":"* Added ECSx.ClientEvents: ephemeral components created by client processes to communicate user input/interaction with the ECSx backend  \n  * ECSx.QueryError renamed to ECSx.MultipleResultsError","ref":"changelog.html#v0-3-1-2023-01-12"},{"type":"extras","title":"v0.3.0 (2023-01-03) - Changelog","doc":"* Components are now stored as key-value pairs  \n  * Component values now require a type declaration which is checked on insertions  \n  * Simplified API for working with components  \n  * Aspects have been renamed to Component Types  \n  * Added Tags: boolean component types which don't store any value  \n  * Component `table_type` now toggled via `:unique` flag","ref":"changelog.html#v0-3-0-2023-01-03"},{"type":"extras","title":"v0.2.0 (2022-08-26) - Changelog","doc":"* New Query API for fetching Components  \n  * Improved generators to better handle code injection  \n  * Generators now raise helpful error messages when missing arguments","ref":"changelog.html#v0-2-0-2022-08-26"},{"type":"extras","title":"v0.1.1 (2022-07-21) - Changelog","doc":"* Setup task `mix ecsx.setup` no longer generates sample modules  \n  * Added option `mix ecsx.setup --no-folders` to prevent generating folders during setup  \n  * Added guides and other documentation","ref":"changelog.html#v0-1-1-2022-07-21"},{"type":"extras","title":"v0.1.0 (2022-07-15) - Changelog","doc":"Initial release","ref":"changelog.html#v0-1-0-2022-07-15"},{"type":"extras","title":"Upgrade Guide","doc":"# Upgrade Guide","ref":"upgrade_guide.html"},{"type":"extras","title":"0.4 to 0.5 - Upgrade Guide","doc":"Non-unique Component types are no longer allowed.  Setting the `:unique` option from within a Component\nmodule now has no effect.  If you are currently using non-unique Component types in your application,\nyou must replace them with Entities as described in the [one_to_many guide](ecs_design.html#one-to-many-associations).\n\nComponent read/write changes:\n\n  * `MyComponent.get_one/1` should be renamed to `MyComponent.get/1`","ref":"upgrade_guide.html#0-4-to-0-5"},{"type":"extras","title":"0.3.x to 0.4 - Upgrade Guide","doc":"In `manager.ex`:\n\n  * `setup do` should be changed to `def startup do`\n  * Remove the `:tick_rate` option if it is set\n  * If the `:tick_rate` option was not the default of 20, add `config :ecsx, tick_rate: n` to your `config.exs`, where n is your chosen tick rate\n\nIn `application.ex`:\n\n  * If the list of children in `start/2` contains `ECSx.ClientEvents` or `YourApp.Manager`, remove them\n\nComponent read/write changes:\n\n  * Any use of `MyComponent.get_one(entity)` where `nil` was a possibility, should be replaced with `MyComponent.get_one(entity, nil)`\n  * Any use of `MyComponent.add(entity, value)` to update the value of a unique component, should be replaced with `MyComponent.update(entity, value)`.  If the `add/2` call was used in a way where sometimes it would create new components, and other times update those components, you will need to separate the two cases to use `add/2` only for the initial creation, and then `update/2` for all subsequent updates.","ref":"upgrade_guide.html#0-3-x-to-0-4"},{"type":"extras","title":"ECS Design","doc":"# ECS Design","ref":"ecs_design.html"},{"type":"extras","title":"Entities and Components - ECS Design","doc":"Everything in your application is an Entity, but in ECS you won't work with these\nEntities directly - instead you will work with the individual attributes that an Entity\nmight have.  These attributes are given to an Entity by creating a Component, which holds,\nat minimum, the Entity's unique ID, but also can store a value.  For example:\n\n* You're running a 2-dimensional simulation of cars on a highway\n* Each car gets its own `entity_id` e.g. `123`\n* If the car with ID `123` is blue, we give it a `Color` Component with value `\"blue\"`\n* If the same car is moving west at 60mph, we might model this with a `Direction` Component with value `\"west\"` and a `Speed` Component with value `60`\n* The car would also have Components such as `XCoordinate` and `YCoordinate` to locate it\n  on the map","ref":"ecs_design.html#entities-and-components"},{"type":"extras","title":"Systems - ECS Design","doc":"Once your Entities are modeled using Components, you'll create Systems to operate on them.\nFor example:\n\n* Entities with `Speed` Components should have their locations regularly updated according to the speed and direction\n* We can create a `Move` System which reads the `Speed` and `Direction` Components, calculates how far the car has moved since the last server tick, and updates the Entity's `XCoordinate` and/or `YCoordinate` Component accordingly.\n* The System will run every tick, only considering Entities which have a `Speed` Component","ref":"ecs_design.html#systems"},{"type":"extras","title":"one-to-many associations - ECS Design","doc":"At some point, you might find yourself thinking of adding multiple Components of the same type to a\nsingle Entity.  We'll call this a \"one-to-many\" association - as in, one Entity, many Components.\nFor example:\n\nLet's say you have a fantasy game where the hero wields a weapon in one hand, and a shield in the\nother hand.  At first, you create a Component for each\n\n```elixir\nWeapon.add(hero_entity, \"Longsword\")\nShield.add(hero_entity, \"Buckler\")\n```\n\nand move on to other features.  However, later on you decide to implement a dual-wielding mechanic\nwhere the hero can wield two swords at a time.  You briefly consider creating a new Component type\ncalled `OffhandWeapon` to go alongside the primary `Weapon`, but then remember that eventually the\nhero will encounter monsters with more than two arms!  Also there is a feature request for weapons\nto be magically enchanted, and eventually you'd also like equipment to lose durability over time and\nrequire repairs at the blacksmith.  So, creating a new Component type for each weapon slot is only\na temporary workaround which is not a very robust solution.\n\nThe ideal solution here is to think about the weapons not as Components, but as separate Entities.\nWhen the hero equips a sword, create a new entity reference for that sword, and reference back\nto the hero entity with one of the sword's Components.\n\n```elixir\nsword_entity = Ecto.UUID.generate()\nDescription.add(sword_entity, \"Longsword\")\nEquippedBy.add(sword_entity, hero_entity)\n```\n\nNow if the hero gets a second sword, we can repeat the process:\n\n```elixir\nanother_sword_entity = Ecto.UUID.generate()\nDescription.add(another_sword_entity, \"Shortsword\")\nEquippedBy.add(another_sword_entity, hero_entity)\n```\n\nFetching a list of weapons equipped by the hero can then be done with `EquippedBy.search(hero_entity)`\n\nTo implement weapon durability:\n\n```elixir\nDurability.add(sword_entity, 150)\nDurability.add(another_sword_entity, 75)\n```\n\nImplementing magic enchantments presents the same situation:  we could add a Component to the sword,\nbut this only works for one simple enchantment per weapon.  In order to allow multiple enchantments\nper weapon, with arbitrary enchantment complexity, we should think about each enchantment as an\nEntity.\n\n```elixir\nenchantment_entity = Ecto.UUID.generate()\nDescription.add(enchantment_entity, \"Firaga\")\nEnchantTarget.add(enchantment_entity, sword_entity)\n```","ref":"ecs_design.html#one-to-many-associations"},{"type":"extras","title":"Common Caveats","doc":"# Common Caveats\n\nThe ECSx API has been carefully designed to avoid common Elixir pitfalls and encourage efficient architectural patterns in your application.  However, there are still some opportunities for bad patterns to sneak in and destabilize performance.  This guide will list known problems and how to avoid them.","ref":"common_caveats.html"},{"type":"extras","title":"Database Queries - Common Caveats","doc":"It is expected that most applications will use a database at some point or another.  However, when dealing with `ECSx.System`s, which run many times per second, most databases are too slow to keep up as the load grows.  Each server tick has a deadline to finish its work, and if it falls behind, there will be lag, unstable performance, and eventually game crashes.\n\n> #### Therefore: {: .error}\n>\n> You should never query the database from within a system\n\nInstead:  use `ECSx.Manager.setup/1` to read the necessary data from the database at startup, and create components with it.  Components are stored in memory, allowing the quick reads and writes which are required for system logic.\n\nExample:\n\n```elixir\ndefmodule MyApp.Manager do\n  use ECSx.Manager\n\n  alias MyApp.Components.Height\n  alias MyApp.Components.XPosition\n  alias MyApp.Components.YPosition\n  alias MyApp.Trees\n  alias MyApp.Trees.Tree\n\n  setup do\n    for %Tree{id: id, x: x, y: y, height: height} <- Trees.my_db_query() do\n        XPosition.add(id, x)\n        YPosition.add(id, y)\n        Height.add(id, height)\n    end\n  end\n  ...\nend\n```\n\nThen when our systems need to work with the height or position of trees, we use the `ECSx.Component` API instead of querying the database again.","ref":"common_caveats.html#database-queries"},{"type":"extras","title":"External Requests - Common Caveats","doc":"Database queries are just one example of a slow call which can hold up the game systems.  Any other request to a service outside your application will likely be too slow to be used in system logic.\n\nInstead:\n\n  * If you only need to make the request once upon initialization, use `ECSx.Manager.setup/1` as shown above\n  * If the request is made once, but triggered by input to some client process, that process should make the request (or spawn a `Task` for it)\n  * If the request should happen regularly, create a new [`GenServer`](https://hexdocs.pm/elixir/GenServer.html#module-receiving-regular-messages) (don't forget to add it to your app's supervision tree)\n  * If you have more than one external request happening regularly, this is a good use case for the [`Oban`](https://hexdocs.pm/oban/Oban.html) library\n  * Remember that client processes (including GenServers, LiveViews, and Oban) have read-only access\n  to components, and must use `ECSx.ClientEvents` for writes","ref":"common_caveats.html#external-requests"},{"type":"extras","title":"search/1 Without Index - Common Caveats","doc":"`ECSx.Component.search/1` will scan all Components of the given type to find matches.  This can be\n  OK if the quantity of Components of that type is small, or if it is just a one-time search.  But\n  if you are searching every tick within a System, through a large list of Components, this can become\n  a performance concern.  The solution is to set the `index: true` option, which will drastically\n  improve search performance.","ref":"common_caveats.html#search-1-without-index"},{"type":"extras","title":"Initial Setup","doc":"# Initial Setup\n\nTo demonstrate ECSx in a real-time application, we're going to make a game where each player will control a ship, which can sail around the map, and will attack enemies if they come too close.\n\n> Note:  This guide will get you up-and-running with a working game, but it is intentionally generic.  Feel free to experiment with altering details from this implementation to customize your own game.\n\n* First, ensure you have installed [Elixir](https://elixir-lang.org/install.html) and [Phoenix](https://hexdocs.pm/phoenix/installation.html) 1.7+.\n* Create the application by running `mix phx.new ship`\n* Run `mix ecto.create` to initialize the database\n* Add `{:ecsx, \"~> 0.5\"}` to your `mix.exs` deps\n* Run `mix deps.get`\n* Run `mix ecsx.setup`","ref":"initial_setup.html"},{"type":"extras","title":"Backend Basics","doc":"# Backend Basics","ref":"backend_basics.html"},{"type":"extras","title":"Defining Component Types - Backend Basics","doc":"First let's consider the basic properties of a ship:\n\n  * Hull Points:  How much damage can it take before it is destroyed\n  * Armor Rating:  How much is each incoming attack reduced by the ship's defenses\n  * Attack Damage:  How much damage does its weapon deal to enemies\n  * Attack Range:  How close must enemies get before the weapon can attack\n  * Attack Speed:  How much time must you wait in-between attacks\n  * X Position:  The horizontal position of the ship\n  * Y Position:  The vertical position of the ship\n  * X Velocity:  The speed at which the ship is moving, horizontally\n  * Y Velocity:  The speed at which the ship is moving, vertically\n\nWe'll start by creating `integer` component types for each one of these, except AttackSpeed, which will use `float`:\n\n    $ mix ecsx.gen.component HullPoints integer\n    $ mix ecsx.gen.component ArmorRating integer\n    $ mix ecsx.gen.component AttackDamage integer\n    $ mix ecsx.gen.component AttackRange integer\n    $ mix ecsx.gen.component XPosition integer\n    $ mix ecsx.gen.component YPosition integer\n    $ mix ecsx.gen.component XVelocity integer\n    $ mix ecsx.gen.component YVelocity integer\n    $ mix ecsx.gen.component AttackSpeed float\n\nFor now, this is all we need to do.  The ECSx generator has automatically set you up with modules for each component type, complete with a simple interface for handling the components.  We'll see this in action soon.","ref":"backend_basics.html#defining-component-types"},{"type":"extras","title":"Our First System - Backend Basics","doc":"Having set up the component types which will model our game data, let's think about the Systems which will organize game logic.  What makes our game work?\n\n  * Ships change position based on velocity\n  * Ships target other ships for attack when they are within range\n  * Ships with valid targets should attack the target, reducing its hull points\n  * Ships with zero or less hull points are destroyed\n  * Players change the velocity of their ship using an input device\n  * Players can see a display of the area around their ship\n\nLet's start with changing position based on velocity.  We'll call it `Driver`:\n\n    $ mix ecsx.gen.system Driver\n\nHead over to the generated file `lib/ship/systems/driver.ex` and we'll add some code:\n\n```elixir\ndefmodule Ship.Systems.Driver do\n  ...\n  @behaviour ECSx.System\n\n  alias Ship.Components.XPosition\n  alias Ship.Components.YPosition\n  alias Ship.Components.XVelocity\n  alias Ship.Components.YVelocity\n\n  @impl ECSx.System\n  def run do\n    for {entity, x_velocity} <- XVelocity.get_all() do\n      x_position = XPosition.get(entity)\n      new_x_position = x_position + x_velocity\n      XPosition.update(entity, new_x_position)\n    end\n\n    # Once the x-values are updated, do the same for the y-values\n    for {entity, y_velocity} <- YVelocity.get_all() do\n      y_position = YPosition.get(entity)\n      new_y_position = y_position + y_velocity\n      YPosition.update(entity, new_y_position)\n    end\n  end\nend\n```\n\nNow whenever a ship gains velocity, this system will update the position accordingly over time.  Keep in mind that the velocity is relative to the server's tick rate, which by default is 20.  This means the unit of measurement is \"game units per 1/20th of a second\".\n\nFor example, if you want the speed to move from XPosition 0 to XPosition 100 in one second, you divide the distance 100 by the tick rate 20, to see that an XVelocity of 5 is appropriate.  The tick rate can be changed in `config/config.ex` and fetched at runtime by calling `ECSx.tick_rate/0`.","ref":"backend_basics.html#our-first-system"},{"type":"extras","title":"Targeting & Attacking - Backend Basics","doc":"Next let's move on to a more complicated part of the game - attacking.  We'll start by considering the conditions which must be met in order to attack a given target:\n\n  * Target must be a ship\n  * Target must be within your ship's attack range\n  * You must not have attacked too recently (based on attack speed)\n\nFor each of these conditions, we want to use the presence or absence of a component as the signal to a system that action is to be taken.  For example, in the Driver system, these were the Velocity components - for each Velocity component, we made a Position update.  \n\nFirst, for determining whether a given entity is a ship, we will simply use the existing HullPoints component, because only ships will have HullPoints.\n\nSecond, for confirming the attack range, we'll make a new component type SeekingTarget which will signal to a Targeting system that a ship's proximity to other ships must be continuously calculated until a valid target is found.  Then another new component type AttackTarget will replace SeekingTarget, signaling to the Targeting system that we no longer need to check for new targets.  Instead, an Attacking system will detect the AttackTarget and handle the final step of the attacking process.\n\nThe final attack requirement is that after a successful attack, the ship's weapon must wait for a cooldown period, based on the attack speed.  To model this cooldown period, we will create an AttackCooldown component type, which will store the time at which the cooldown expires.\n\nWith this plan in place, let's go ahead and create the component types, starting with SeekingTarget.  Since the presence of this component alone fulfills its purpose, without the need to store additional data, this is the appropriate use-case for a `Tag`:\n\n    $ mix ecsx.gen.tag SeekingTarget\n\nOnce a target is found, the `AttackTarget` component will be needed, and this time a `Tag` will not be enough, because we need to store the ID of the target.  Likewise with `AttackCooldown`, which must store the timestamp of the cooldown's expiration.\n\n    $ mix ecsx.gen.component AttackTarget binary\n    $ mix ecsx.gen.component AttackCooldown datetime\n\n> Note:  In our case, we're using binary IDs to represent Entities, and Elixir `DateTime` structs for cooldown expirations.  If you're planning on using different types, such as integer IDs for entities, or storing timestamps as integers, simply adjust the parameters accordingly.\n\nBefore we set up the systems, let's make a helper module for storing any shared mathematical logic.  In particular, we'll need a function for calculating the distance between two entities.  This will come in handy for several systems in the future.\n\n```elixir\ndefmodule Ship.SystemUtils do\n  @moduledoc \"\"\"\n  Useful math functions used by multiple systems.\n  \"\"\"\n\n  alias Ship.Components.XPosition\n  alias Ship.Components.YPosition\n\n  def distance_between(entity_1, entity_2) do\n    x_1 = XPosition.get(entity_1)\n    x_2 = XPosition.get(entity_2)\n    y_1 = YPosition.get(entity_1)\n    y_2 = YPosition.get(entity_2)\n\n    x = abs(x_1 - x_2)\n    y = abs(y_1 - y_2)\n\n    :math.sqrt(x ** 2 + y ** 2)\n  end\nend\n```\n\nNow we're onto the Targeting system, which operates only on entities with the SeekingTarget component, checking the distance to all other ships, and comparing them to the entity's attack range.  When an enemy ship is found to be within range, we can remove SeekingTarget and replace it with an AttackTarget:\n\n    $ mix ecsx.gen.system Targeting\n\n```elixir\ndefmodule Ship.Systems.Targeting do\n  ...\n  @behaviour ECSx.System\n\n  alias Ship.Components.AttackRange\n  alias Ship.Components.AttackTarget\n  alias Ship.Components.HullPoints\n  alias Ship.Components.SeekingTarget\n  alias Ship.SystemUtils\n\n  @impl ECSx.System\n  def run do\n    entities = SeekingTarget.get_all()\n\n    Enum.each(entities, &attempt_target/1)\n  end\n\n  defp attempt_target(self) do\n    case look_for_target(self) do\n      nil -> :noop\n      {target, _hp} -> add_target(self, target)\n    end\n  end \n\n  defp look_for_target(self) do\n    # For now, we're assuming anything which has HullPoints can be attacked\n    HullPoints.get_all()\n    # ... except your own ship!\n    |> Enum.reject(fn {possible_target, _hp} -> possible_target == self end)\n    |> Enum.find(fn {possible_target, _hp} ->\n      distance_between = SystemUtils.distance_between(possible_target, self)\n      range = AttackRange.get(self)\n\n      distance_between   AttackRange.get(self) ->\n        # If the target ever leaves our attack range, we want to remove the AttackTarget\n        # and begin searching for a new one.\n        AttackTarget.remove(self)\n        SeekingTarget.add(self)\n\n      AttackCooldown.exists?(self) ->\n        # We're still within range, but waiting on the cooldown\n        :noop\n\n      :otherwise ->\n        deal_damage(self, target)\n        add_cooldown(self)\n    end\n  end\n\n  defp deal_damage(self, target) do\n    attack_damage = AttackDamage.get(self)\n    # Assuming one armor rating always equals one damage\n    reduction_from_armor = ArmorRating.get(target)\n    final_damage_amount = attack_damage - reduction_from_armor\n\n    target_current_hp = HullPoints.get(target)\n    target_new_hp = target_current_hp - final_damage_amount\n\n    HullPoints.update(target, target_new_hp)\n  end\n\n  defp add_cooldown(self) do\n    now = DateTime.utc_now()\n    ms_between_attacks = calculate_cooldown_time(self)\n    cooldown_until = DateTime.add(now, ms_between_attacks, :millisecond)\n\n    AttackCooldown.add(self, cooldown_until)\n  end\n\n  # We're going to model AttackSpeed with a float representing attacks per second.\n  # The goal here is to convert that into milliseconds per attack.\n  defp calculate_cooldown_time(self) do\n    attacks_per_second = AttackSpeed.get(self)\n    seconds_per_attack = 1 / attacks_per_second\n\n    ceil(seconds_per_attack * 1000)\n  end\nend\n```\n\nPhew, that was a lot!  But we're still using the same basic concepts:  `get_all/0` to fetch the list of all relevant entities, then `get/1` and `exists?/1` to check specific attributes of the entities, `add/2` for creating new components, and `update/2` for overwriting existing ones.  We're also starting to see the use of `remove/1` for excluding an entity from game logic which is no longer necessary.","ref":"backend_basics.html#targeting-attacking"},{"type":"extras","title":"Cooldowns - Backend Basics","doc":"Our attacking system will add a cooldown with an expiration timestamp, but the next step is to ensure the cooldown component is removed from the entity once the time is reached, so it can attack again.  For that, we'll create a `CooldownExpiration` system:\n\n    $ mix ecsx.gen.system CooldownExpiration\n\n```elixir\ndefmodule Ship.Systems.CooldownExpiration do\n  ...\n  @behaviour ECSx.System\n\n  alias Ship.Components.AttackCooldown\n\n  @impl ECSx.System\n  def run do\n    now = DateTime.utc_now()\n    cooldowns = AttackCooldown.get_all()\n      \n    Enum.each(cooldowns, &remove_when_expired(&1, now))\n  end\n\n  defp remove_when_expired({entity, timestamp}, now) do\n    case DateTime.compare(now, timestamp) do\n      :lt -> :noop\n      _ -> AttackCooldown.remove(entity)\n    end\n  end\nend\n```\n\nThis system will check the cooldowns on each game tick, removing them as soon as the expiration time is reached.","ref":"backend_basics.html#cooldowns"},{"type":"extras","title":"Death & Destruction - Backend Basics","doc":"Next let's handle what happens when a ship has its HP reduced to zero or less:\n\n    $ mix ecsx.gen.component DestroyedAt datetime\n\n    $ mix ecsx.gen.system Destruction\n\n```elixir\ndefmodule Ship.Systems.Destruction do\n  ...\n  @behaviour ECSx.System\n\n  alias Ship.Components.ArmorRating\n  alias Ship.Components.AttackCooldown\n  alias Ship.Components.AttackDamage\n  alias Ship.Components.AttackRange\n  alias Ship.Components.AttackSpeed\n  alias Ship.Components.AttackTarget\n  alias Ship.Components.DestroyedAt\n  alias Ship.Components.HullPoints\n  alias Ship.Components.SeekingTarget\n  alias Ship.Components.XPosition\n  alias Ship.Components.XVelocity\n  alias Ship.Components.YPosition\n  alias Ship.Components.YVelocity\n\n  @impl ECSx.System\n  def run do\n    ships = HullPoints.get_all()\n\n    Enum.each(ships, fn {entity, hp} ->\n      if hp <= 0, do: destroy(entity)\n    end)\n  end\n\n  defp destroy(ship) do\n    ArmorRating.remove(ship)\n    AttackCooldown.remove(ship)\n    AttackDamage.remove(ship)\n    AttackRange.remove(ship)\n    AttackSpeed.remove(ship)\n    AttackTarget.remove(ship)\n    HullPoints.remove(ship)\n    SeekingTarget.remove(ship)\n    XPosition.remove(ship)\n    XVelocity.remove(ship)\n    YPosition.remove(ship)\n    YVelocity.remove(ship)\n\n    # when a ship is destroyed, other ships should stop targeting it\n    untarget(ship)\n\n    DestroyedAt.add(ship, DateTime.utc_now())\n  end\n\n  defp untarget(target) do\n    for ship <- AttackTarget.search(target) do\n      AttackTarget.remove(ship)\n      SeekingTarget.add(ship)\n    end\n  end\nend\n```\n\nIn this example we remove all the components the entity might have, then add a new DestroyedAt component with the current timestamp.  If we wanted some components to persist - such as the position and/or velocity, so the wreckage could still be visible on the player displays - we could keep them around and possibly have another system clean them up later on.  Likewise if there were other components to add, such as a `RespawnTimer` or `FinalScore`, we could add them here as well.","ref":"backend_basics.html#death-destruction"},{"type":"extras","title":"Initializing Component Data - Backend Basics","doc":"By now you might be wondering \"How did those components get created in the first place?\"  We have code for adding `AttackCooldown` and `DestroyedAt`, when needed, but the basic components for the ships still need to be added before the game can even start.  For that, we'll check out `lib/ship/manager.ex`:\n\n```elixir\ndefmodule Ship.Manager do\n  ...\n  use ECSx.Manager\n\n  def setup do\n    ...\n  end\n\n  def startup do\n    ...\n  end\n\n  def components do\n    ...\n  end\n\n  def systems do\n    ...\n  end\nend\n```\n\nThis module holds three critical pieces of data - component setup, a list of every valid component type, and a list of each game system in the order they are to be run.  Let's create some ship components inside the `startup` block:\n\n```elixir\ndef startup do\n  for _ships <- 1..40 do\n    # First generate a unique ID to represent the new entity\n    entity = Ecto.UUID.generate()\n\n    # Then use that ID to create the components which make up a ship    \n    Ship.Components.ArmorRating.add(entity, 0)\n    Ship.Components.AttackDamage.add(entity, 5)\n    Ship.Components.AttackRange.add(entity, 10)\n    Ship.Components.AttackSpeed.add(entity, 1.05)\n    Ship.Components.HullPoints.add(entity, 50)\n    Ship.Components.SeekingTarget.add(entity)\n    Ship.Components.XPosition.add(entity, Enum.random(1..100))\n    Ship.Components.YPosition.add(entity, Enum.random(1..100))\n    Ship.Components.XVelocity.add(entity, 0)\n    Ship.Components.YVelocity.add(entity, 0)\n  end\nend\n```\n\nNow whenever the server starts, there will be forty ships set up and ready to go.","ref":"backend_basics.html#initializing-component-data"},{"type":"extras","title":"Web Frontend with LiveView","doc":"# Web Frontend with LiveView\n\nSince we're using Phoenix, we can take advantage of the many features it brings for building a web interface.","ref":"web_frontend_liveview.html"},{"type":"extras","title":"Player Auth - Web Frontend with LiveView","doc":"When it comes to player auth, there are two sides to the coin:  authentication (AuthN) and authorization (AuthZ).  The former refers to verifying the identity of a player (and will be our primary focus, for now), while the latter refers to checking whether a user has permission to take a restricted action.\n\nPhoenix comes with an [AuthN generator](https://hexdocs.pm/phoenix/Mix.Tasks.Phx.Gen.Auth.html) built-in, which should be more than enough for our needs:\n\n    $ mix phx.gen.auth Players Player players --binary-id\n    $ mix deps.get\n    $ mix ecto.migrate\n\nThis will expect players to register an email and password, which will be used to log in.  A unique ID will also be created for each player upon registration, allowing us to begin thinking of players as entities.  However, we can't just take the player input and start creating components with it - only systems can create components.  Instead, we'll use a special component type provided for this purpose: `ECSx.ClientEvents`.","ref":"web_frontend_liveview.html#player-auth"},{"type":"extras","title":"Client Input via LiveView - Web Frontend with LiveView","doc":"First consider the goals for our frontend:\n\n  * Authenticate the player and hold player ID\n  * Spawn the player's ship upon connection (writes components)\n  * Hold the coordinates for the player's ship\n  * Hold the coordinates for enemy ships\n  * Validate user input to move the ship (writes components)\n\nWhen we need to write components, `ECSx.ClientEvents` will be our line of communication from the frontend to the backend.  \n\nLet's create `/lib/ship_web/live/game_live.ex` and put it to use:\n\n```elixir\ndefmodule ShipWeb.GameLive do\n  use ShipWeb, :live_view\n\n  alias Ship.Components.HullPoints\n  alias Ship.Components.XPosition\n  alias Ship.Components.YPosition\n\n  def mount(_params, %{\"player_token\" => token} = _session, socket) do\n    # This context function was generated by phx.gen.auth\n    player = Ship.Players.get_player_by_session_token(token)\n\n    socket =\n      socket\n      |> assign(player_entity: player.id)\n      # Keeping a set of currently held keys will allow us to prevent duplicate keydown events\n      |> assign(keys: MapSet.new())\n      # We don't know where the ship will spawn, yet\n      |> assign(x_coord: nil, y_coord: nil, current_hp: nil)\n\n    # We don't want these calls to be made on both the initial static page render and again after\n    # the LiveView is connected, so we wrap them in `connected?/1` to prevent duplication\n    if connected?(socket) do\n      ECSx.ClientEvents.add(player.id, :spawn_ship)\n      :timer.send_interval(50, :load_player_info)\n    end\n\n    {:ok, socket}\n  end\n\n  def handle_info(:load_player_info, socket) do\n    # This will run every 50ms to keep the client assigns updated\n    x = XPosition.get(socket.assigns.player_entity)\n    y = YPosition.get(socket.assigns.player_entity)\n    hp = HullPoints.get(socket.assigns.player_entity)\n\n    {:noreply, assign(socket, x_coord: x, y_coord: y, current_hp: hp)}\n  end\n\n  def handle_event(\"keydown\", %{\"key\" => key}, socket) do\n    if MapSet.member?(socket.assigns.keys, key) do\n      # Already holding this key - do nothing\n      {:noreply, socket}\n    else\n      # We only want to add a client event if the key is defined by the `keydown/1` helper below\n      maybe_add_client_event(socket.assigns.player_entity, key, &keydown/1)\n      {:noreply, assign(socket, keys: MapSet.put(socket.assigns.keys, key))}\n    end\n  end\n\n  def handle_event(\"keyup\", %{\"key\" => key}, socket) do\n    # We don't have to worry about duplicate keyup events\n    # But once again, we will only add client events for keys that actually do something\n    maybe_add_client_event(socket.assigns.player_entity, key, &keyup/1)\n    {:noreply, assign(socket, keys: MapSet.delete(socket.assigns.keys, key))}\n  end\n\n  defp maybe_add_client_event(player_entity, key, fun) do\n    case fun.(key) do\n      :noop -> :ok\n      event -> ECSx.ClientEvents.add(player_entity, event)\n    end\n  end\n\n  defp keydown(key) when key in ~w(w W ArrowUp), do: {:move, :north}\n  defp keydown(key) when key in ~w(a A ArrowLeft), do: {:move, :west}\n  defp keydown(key) when key in ~w(s S ArrowDown), do: {:move, :south}\n  defp keydown(key) when key in ~w(d D ArrowRight), do: {:move, :east}\n  defp keydown(_key), do: :noop\n\n  defp keyup(key) when key in ~w(w W ArrowUp), do: {:stop_move, :north}\n  defp keyup(key) when key in ~w(a A ArrowLeft), do: {:stop_move, :west}\n  defp keyup(key) when key in ~w(s S ArrowDown), do: {:stop_move, :south}\n  defp keyup(key) when key in ~w(d D ArrowRight), do: {:stop_move, :east}\n  defp keyup(_key), do: :noop\n\n  def render(assigns) do\n    ~H\"\"\"\n     \n       Player ID: <%= @player_entity %> \n       Player Coords: <%= inspect({@x_coord, @y_coord}) %> \n       Hull Points: <%= @current_hp %> \n     \n    \"\"\"\n  end\nend\n```","ref":"web_frontend_liveview.html#client-input-via-liveview"},{"type":"extras","title":"Handling Client Events - Web Frontend with LiveView","doc":"Finally, spin up a new system for handling the events:\n\n    $ mix ecsx.gen.system ClientEventHandler\n\n```elixir\ndefmodule Ship.Systems.ClientEventHandler do\n  ...\n  @behaviour ECSx.System\n\n  alias Ship.Components.ArmorRating\n  alias Ship.Components.AttackDamage\n  alias Ship.Components.AttackRange\n  alias Ship.Components.AttackSpeed\n  alias Ship.Components.HullPoints\n  alias Ship.Components.SeekingTarget\n  alias Ship.Components.XPosition\n  alias Ship.Components.XVelocity\n  alias Ship.Components.YPosition\n  alias Ship.Components.YVelocity\n\n  @impl ECSx.System\n  def run do\n    client_events = ECSx.ClientEvents.get_and_clear()\n\n    Enum.each(client_events, &process_one/1)\n  end\n\n  defp process_one({player, :spawn_ship}) do\n    # We'll give player ships better stats than the enemy ships\n    # (otherwise the game would be very short!)\n    ArmorRating.add(player, 2)\n    AttackDamage.add(player, 6)\n    AttackRange.add(player, 15)\n    AttackSpeed.add(player, 1.2)\n    HullPoints.add(player, 75)\n    SeekingTarget.add(player)\n    XPosition.add(player, Enum.random(1..100))\n    YPosition.add(player, Enum.random(1..100))\n    XVelocity.add(player, 0)\n    YVelocity.add(player, 0)\n  end\n\n  # Note Y movement will use screen position (increasing Y goes south)\n  defp process_one({player, {:move, :north}}), do: YVelocity.update(player, -1)\n  defp process_one({player, {:move, :south}}), do: YVelocity.update(player, 1)\n  defp process_one({player, {:move, :east}}), do: XVelocity.update(player, 1)\n  defp process_one({player, {:move, :west}}), do: XVelocity.update(player, -1)\n\n  defp process_one({player, {:stop_move, :north}}), do: YVelocity.update(player, 0)\n  defp process_one({player, {:stop_move, :south}}), do: YVelocity.update(player, 0)\n  defp process_one({player, {:stop_move, :east}}), do: XVelocity.update(player, 0)\n  defp process_one({player, {:stop_move, :west}}), do: XVelocity.update(player, 0)\nend\n```\n\nNotice how the LiveView client can write to `ECSx.ClientEvents`, while the system handles and also clears the events.  This ensures that we don't process the same event twice, nor will any events get \"lost\" and not processed.","ref":"web_frontend_liveview.html#handling-client-events"},{"type":"extras","title":"Creating a Phoenix Route - Web Frontend with LiveView","doc":"Head into `router.ex` and look for the new scope which uses `:require_authenticated_player`.  We're going to add a new route for our game interface:\n\n```elixir\nscope \"/\", ShipWeb do\n  pipe_through [:browser, :require_authenticated_player]\n\n  live_session :require_authenticated_player,\n    on_mount: [{ShipWeb.PlayerAuth, :ensure_authenticated}] do\n    live \"/game\", GameLive\n    ...\n  end\nend\n```\n\nNow we can run\n\n    $ iex -S mix phx.server\n\nand go to `localhost:4000/game` to test the input.  Once you are logged in, wait for the player coords to display (this will be the indicator that your ship has spawned), and try moving around with `WASD` or arrow keys!","ref":"web_frontend_liveview.html#creating-a-phoenix-route"},{"type":"extras","title":"Loading Screen - Web Frontend with LiveView","doc":"You might notice that while the ship is spawning, the Player Coords and Hull Points don't display properly - this isn't a major issue now, but once our coordinates are being used by a more sophisticated display, this will not be acceptable.  What we need is a loading screen to show the user until the necessary data is properly loaded.\n\nFirst, let's create a new `ECSx.Tag` to mark when a player's ship has finished spawning:\n\n    $ mix ecsx.gen.tag PlayerSpawned\n\nThen we'll add this tag at the end of the `:spawn_ship` client event\n\n```elixir\ndefmodule Ship.Systems.ClientEventHandler do\n  ...\n  alias Ship.Components.PlayerSpawned\n  ...\n  defp process_one({player, :spawn_ship}) do\n    ...\n    PlayerSpawned.add(player)\n  end\n  ...\nend\n```\n\nNow we'll update our LiveView to use a new `@loading` assign which is initially set to `true`, then set to `false` after the ship is spawned and the data is loaded for the first time.\n\nReplace both the current `mount` and `handle_info` functions with the below functions, and replace the existing `render` function\nwith the new `render` function below \n\n```elixir\ndefmodule ShipWeb.GameLive do\n  ...\n  alias Ship.Components.PlayerSpawned\n  ...\n  def mount(_params, %{\"player_token\" => token} = _session, socket) do\n    player = Ship.Players.get_player_by_session_token(token)\n\n    socket =\n      socket\n      |> assign(player_entity: player.id)\n      |> assign(keys: MapSet.new())\n      # This gets its own helper in case we need to return to this state again later\n      |> assign_loading_state()\n\n    if connected?(socket) do\n      ECSx.ClientEvents.add(player.id, :spawn_ship)\n      # The first load will now have additional responsibilities\n      send(self(), :first_load)\n    end\n    \n\n    {:ok, socket}\n  end\n\n  defp assign_loading_state(socket) do\n    assign(socket,\n      x_coord: nil,\n      y_coord: nil,\n      current_hp: nil,\n      # This new assign will control whether the loading screen is shown\n      loading: true\n    )\n  end\n\n  def handle_info(:first_load, socket) do\n    # Don't start fetching components until after spawn is complete!\n    :ok = wait_for_spawn(socket.assigns.player_entity)\n\n    socket =\n      socket\n      |> assign_player_ship()\n      |> assign(loading: false)\n\n    # We want to keep up-to-date on this info\n    :timer.send_interval(50, :refresh)\n\n    {:noreply, socket}\n  end\n\n  def handle_info(:refresh, socket) do\n    {:noreply, assign_player_ship(socket)}\n  end\n\n  defp wait_for_spawn(player_entity) do\n    if PlayerSpawned.exists?(player_entity) do\n      :ok\n    else\n      Process.sleep(10)\n      wait_for_spawn(player_entity)\n    end\n  end\n\n  # Our previous :load_player_info handler becomes a shared helper for the new handlers\n  defp assign_player_ship(socket) do\n    x = XPosition.get(socket.assigns.player_entity)\n    y = YPosition.get(socket.assigns.player_entity)\n    hp = HullPoints.get(socket.assigns.player_entity)\n\n    assign(socket, x_coord: x, y_coord: y, current_hp: hp)\n  end\n\n  def handle_event(\"keydown\", %{\"key\" => key}, socket) do\n  ...\n\n  def render(assigns) do\n    ~H\"\"\"\n     \n      <%= if @loading do %>\n         Loading... \n      <% else %>\n         Player ID: <%= @player_entity %> \n         Player Coords: <%= inspect({@x_coord, @y_coord}) %> \n         Hull Points: <%= @current_hp %> \n      <% end %>\n     \n    \"\"\"\n  end\nend\n```","ref":"web_frontend_liveview.html#loading-screen"},{"type":"extras","title":"Player GUI using SVG - Web Frontend with LiveView","doc":"One of the simplest ways to build a display for web is with SVG.  Each entity can be represented by a single SVG element, which only requires its coordinates.  Then a `viewBox` can zoom the player's display in to show just the local area around their ship.\n\n```elixir\ndefmodule ShipWeb.GameLive do\n  ...\n  def render(assigns) do\n    ~H\"\"\"\n     \n       \n         \n\n        <%= if @loading do %>\n           \n            Loading...\n           \n        <% else %>\n           \n          <%= for {_entity, x, y, image_file} <- @other_ships do %>\n             \n          <% end %>\n           \n            Hull Points: <%= @current_hp %>\n           \n        <% end %>\n       \n     \n    \"\"\"\n  end\nend\n```\n\nWe've added a lot here, so let's go line-by-line:\n\nWe're filling the screen with an [`svg viewBox`](https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/viewBox), which takes four arguments.  The first two - x and y offsets - tell the viewBox what area of the map to focus on, while the latter two - screen width and height - tell it how much to zoom in.  To get the offsets, we'll need to calculate the coordinate pair which should be at the very top-left of the player's view.  This will need to be updated every time the player moves.  The screen width and height (measured by game coordinates) will be assigned on mount and won't change.  The `preserveAspectRatio` has two parts: `xMinYMin` means we define the offset by the top-left coordinate, and `slice` means we're only expecting to show a \"slice\" of the map in our viewBox, not the whole thing.\n\nOur first element will be a simple `rect` (rectangle) with a light-blue fill to cover the entire world map.  This will be the \"background\" - representing the ocean.  The game world will consistently be 100x100, so we can assign a `game_world_size` of 100 for this purpose.\n\nThe loading screen will still use the same viewBox and background, but with only one other element - `text` to display a loading message in the center of the screen.  One curiosity regarding the font size:  we use `1px`, but you'll see later that the text is actually quite large.  This is because the viewBox considers each game \"tile\" to be 1 pixel, and automatically scales these pixels up to a larger size based on the screen width and height compared to the size of the browser window.  So when we say `font: 1px` it means the text will be as tall as one game tile.\n\nOnce the game is finished loading, we'll display three things:  the player's ship, other ships, and the player's current HP.\n\nFor the player's ship, we'll make an `image` element, using the existing x and y coordinates, defining the size as one game tile, and pointing to the player's ship image file.\n\nFor other ships, we'll need a new assign to hold that data - ID and coordinates, at minimum.  Then each one will get an `image` just like the player's ship.\n\nLastly, we'll put an HP display near the top-left corner.\n\nLets first create an `ImageFile` component:\n\n    $ mix ecsx.gen.component ImageFile binary\n\nNext step is to update our LiveView with these new assigns:\n\n```elixir\ndefmodule ShipWeb.GameLive do\n  ...\n  alias Ship.Components.ImageFile\n  ...\n  def mount(_params, %{\"player_token\" => token} = _session, socket) do\n    player = Ship.Players.get_player_by_session_token(token)\n\n    socket =\n      socket\n      |> assign(player_entity: player.id)\n      |> assign(keys: MapSet.new())\n      # These will configure the scale of our display compared to the game world\n      |> assign(game_world_size: 100, screen_height: 30, screen_width: 50)\n      |> assign_loading_state()\n    \n    if connected?(socket) do\n      ECSx.ClientEvents.add(player.id, :spawn_ship)\n      send(self(), :first_load)\n    end\n\n    {:ok, socket}\n  end\n\n  defp assign_loading_state(socket) do\n    assign(socket,\n      x_coord: nil,\n      y_coord: nil,\n      current_hp: nil,\n      player_ship_image_file: nil,\n      other_ships: [],\n      x_offset: 0,\n      y_offset: 0,\n      loading: true\n    )\n  end\n\n  def handle_info(:first_load, socket) do\n    :ok = wait_for_spawn(socket.assigns.player_entity)\n\n    socket =\n      socket\n      |> assign_player_ship()\n      |> assign_other_ships()\n      |> assign_offsets()\n      |> assign(loading: false)\n\n    :timer.send_interval(50, :refresh)\n\n    {:noreply, socket}\n  end\n\n  def handle_info(:refresh, socket) do\n    socket =\n      socket\n      |> assign_player_ship()\n      |> assign_other_ships()\n      |> assign_offsets()\n\n    {:noreply, socket}\n  end\n\n  defp wait_for_spawn(player_entity) do\n    if PlayerSpawned.exists?(player_entity) do\n      :ok\n    else\n      Process.sleep(10)\n      wait_for_spawn(player_entity)\n    end\n  end\n\n  defp assign_player_ship(socket) do\n    x = XPosition.get(socket.assigns.player_entity)\n    y = YPosition.get(socket.assigns.player_entity)\n    hp = HullPoints.get(socket.assigns.player_entity)\n    image = ImageFile.get(socket.assigns.player_entity)\n\n    assign(socket, x_coord: x, y_coord: y, current_hp: hp, player_ship_image_file: image)\n  end\n\n  defp assign_other_ships(socket) do\n    other_ships =\n      Enum.reject(all_ships(), fn {entity, _, _, _} -> entity == socket.assigns.player_entity end)\n\n    assign(socket, other_ships: other_ships)\n  end\n\n  defp all_ships do\n    for {ship, _hp} <- HullPoints.get_all() do\n      x = XPosition.get(ship)\n      y = YPosition.get(ship)\n      image = ImageFile.get(ship)\n      {ship, x, y, image}\n    end\n  end\n\n  defp assign_offsets(socket) do\n    # Note: the socket must already have updated player coordinates before assigning offsets!\n    %{screen_width: screen_width, screen_height: screen_height} = socket.assigns\n    %{x_coord: x, y_coord: y, game_world_size: game_world_size} = socket.assigns\n\n    x_offset = calculate_offset(x, screen_width, game_world_size)\n    y_offset = calculate_offset(y, screen_height, game_world_size)\n\n    assign(socket, x_offset: x_offset, y_offset: y_offset)\n  end\n\n  defp calculate_offset(coord, screen_size, game_world_size) do\n    case coord - div(screen_size, 2) do\n      offset when offset < 0 -> 0\n      offset when offset > game_world_size - screen_size -> game_world_size - screen_size\n      offset -> offset\n    end\n  end\n\n  def handle_event(\"keydown\", %{\"key\" => key}, socket) do \n  ...\nend\n```\n\nNext let's create the `ImageFile` components when a ship is spawned:\n\n```elixir\ndefmodule Ship.Manager do\n  ...\n  def startup do\n    for _ships <- 1..40 do\n      ...\n      Ship.Components.ImageFile.add(entity, \"npc_ship.svg\")\n    end\n  end\n  ...\nend\n```\n\n```elixir\ndefmodule Ship.Systems.ClientEventHandler do\n  ...\n  alias Ship.Components.ImageFile\n  ...\n  defp process_one({player, :spawn_ship}) do\n    ...\n    ImageFile.add(player, \"player_ship.svg\")\n    PlayerSpawned.add(player)\n  end\n  ...\nend\n```\n\nLastly, we'll need the [player_ship.svg](https://raw.githubusercontent.com/ecsx-framework/ship/master/priv/static/images/player_ship.svg) and [npc_ship.svg](https://raw.githubusercontent.com/ecsx-framework/ship/master/priv/static/images/npc_ship.svg) files.  Right-click on the links and save them to `priv/static/images/`, where they will be found by our `~p\"/images/...\"` calls in the LiveView template.\n\nNow running\n\n    $ iex -S mix phx.server\n\nand heading to `localhost:4000/game` should provide a usable game interface to move your ship around, ideally keeping it out of attack range of enemy ships, while remaining close enough for your own ship to attack (remember that we gave the player ship a longer attack range than the enemy ships).","ref":"web_frontend_liveview.html#player-gui-using-svg"},{"type":"extras","title":"Projectile Animations - Web Frontend with LiveView","doc":"Currently the most challenging part of the game is knowing when your ship is attacking, and when it is being attacked.  Let's implement a new feature to make attacks visible to the player(s).  There are several ways to go about this;  we're going to take an approach that showcases ECS design:\n\n  * Instead of an attack immediately dealing damage, it will spawn a cannonball entity\n  * The cannonball entity will have position and velocity components, like ships do\n  * It will also have new components such as `ProjectileTarget` and `ProjectileDamage`\n  * A `Projectile` system will guide it to its target, then destroy the cannonball and deal damage\n  * In our LiveView, we'll create a new assign to hold the locations of projectiles\n  * The new assign will be used to create SVG elements\n  * To help fetch locations for projectiles only, we'll add an `IsProjectile` tag\n\nStart by running the generator commands for our new components, systems, and tag:\n\n    $ mix ecsx.gen.component ProjectileTarget binary\n    $ mix ecsx.gen.component ProjectileDamage integer\n    $ mix ecsx.gen.system Projectile\n    $ mix ecsx.gen.tag IsProjectile\n\nThen we need to update the `Attacking` system to spawn projectiles instead of immediately dealing damage.  We'll replace the existing `deal_damage/2` with a `spawn_projectile/2`:\n\n```elixir\ndefmodule Ship.Systems.Attacking do\n  ...\n  @behaviour ECSx.System\n\n  alias Ship.Components.AttackCooldown\n  alias Ship.Components.AttackDamage\n  alias Ship.Components.AttackRange\n  alias Ship.Components.AttackSpeed\n  alias Ship.Components.AttackTarget\n  alias Ship.Components.ImageFile\n  alias Ship.Components.IsProjectile\n  alias Ship.Components.ProjectileDamage\n  alias Ship.Components.ProjectileTarget\n  alias Ship.Components.SeekingTarget\n  alias Ship.Components.XPosition\n  alias Ship.Components.XVelocity\n  alias Ship.Components.YPosition\n  alias Ship.Components.YVelocity\n  alias Ship.SystemUtils\n  ...\n  defp attack_if_ready({self, target}) do\n    cond do\n      ...\n      :otherwise ->\n        spawn_projectile(self, target)\n        add_cooldown(self)\n    end\n  end\n\n  defp spawn_projectile(self, target) do\n    attack_damage = AttackDamage.get(self)\n    x = XPosition.get(self)\n    y = YPosition.get(self)\n    # Armor reduction should wait until impact to be calculated\n    cannonball_entity = Ecto.UUID.generate()\n\n    IsProjectile.add(cannonball_entity)\n    XPosition.add(cannonball_entity, x)\n    YPosition.add(cannonball_entity, y)\n    XVelocity.add(cannonball_entity, 0)\n    YVelocity.add(cannonball_entity, 0)\n    ImageFile.add(cannonball_entity, \"cannonball.svg\")\n    ProjectileTarget.add(cannonball_entity, target)\n    ProjectileDamage.add(cannonball_entity, attack_damage)\n  end\n  ...\nend\n```\n\nNotice we start the velocity at zero, because the movement will be entirely handled by the `Projectile` system:\n\n```elixir\ndefmodule Ship.Systems.Projectile do\n  ...\n  @behaviour ECSx.System\n\n  alias Ship.Components.ArmorRating\n  alias Ship.Components.HullPoints\n  alias Ship.Components.ImageFile\n  alias Ship.Components.IsProjectile\n  alias Ship.Components.ProjectileDamage\n  alias Ship.Components.ProjectileTarget\n  alias Ship.Components.XPosition\n  alias Ship.Components.XVelocity\n  alias Ship.Components.YPosition\n  alias Ship.Components.YVelocity\n\n  @cannonball_speed 3\n\n  @impl ECSx.System\n  def run do\n    projectiles = IsProjectile.get_all()\n      \n    Enum.each(projectiles, fn projectile ->\n      case ProjectileTarget.get(projectile, nil) do\n        nil ->\n          # The target has already been destroyed\n          destroy_projectile(projectile)\n\n        target ->\n          continue_seeking_target(projectile, target)\n      end\n    end)\n  end\n\n  defp continue_seeking_target(projectile, target) do\n    {dx, dy, distance} = get_distance_to_target(projectile, target)\n\n    case distance do\n      0 ->\n        collision(projectile, target)\n\n      distance when distance / @cannonball_speed <= 1 ->\n        move_directly_to_target(projectile, {dx, dy})\n\n      distance ->\n        adjust_velocity_towards_target(projectile, {distance, dx, dy})\n    end\n  end\n\n  defp get_distance_to_target(projectile, target) do\n    target_x = XPosition.get(target)\n    target_y = YPosition.get(target)\n    target_dx = XVelocity.get(target)\n    target_dy = YVelocity.get(target)\n    target_next_x = target_x + target_dx\n    target_next_y = target_y + target_dy\n\n    x = XPosition.get(projectile)\n    y = YPosition.get(projectile)\n\n    dx = target_next_x - x\n    dy = target_next_y - y\n\n    {dx, dy, ceil(:math.sqrt(dx ** 2 + dy ** 2))}\n  end\n\n  defp collision(projectile, target) do\n    damage_target(projectile, target)\n    destroy_projectile(projectile)\n  end\n\n  defp damage_target(projectile, target) do\n    damage = ProjectileDamage.get(projectile)\n    reduction_from_armor = ArmorRating.get(target)\n    final_damage_amount = damage - reduction_from_armor\n\n    target_current_hp = HullPoints.get(target)\n    target_new_hp = target_current_hp - final_damage_amount\n\n    HullPoints.update(target, target_new_hp)\n  end\n\n  defp destroy_projectile(projectile) do\n    IsProjectile.remove(projectile)\n    XPosition.remove(projectile)\n    YPosition.remove(projectile)\n    XVelocity.remove(projectile)\n    YVelocity.remove(projectile)\n    ImageFile.remove(projectile)\n    ProjectileTarget.remove(projectile)\n    ProjectileDamage.remove(projectile)\n  end\n\n  defp move_directly_to_target(projectile, {dx, dy}) do\n    XVelocity.update(projectile, dx)\n    YVelocity.update(projectile, dy)\n  end\n\n  defp adjust_velocity_towards_target(projectile, {distance, dx, dy}) do\n    # We know what is needed, but we need to slow it down, so its travel\n    # will take more than one tick.  Otherwise the player will not see it!\n    ticks_away = ceil(distance / @cannonball_speed)\n    adjusted_dx = div(dx, ticks_away)\n    adjusted_dy = div(dy, ticks_away)\n\n    XVelocity.update(projectile, adjusted_dx)\n    YVelocity.update(projectile, adjusted_dy)\n  end\nend\n```\n\nNote that we rely on the absence of a `ProjectileTarget` to know that the target is already destroyed.  Currently our `Destruction` system does have an `untarget` feature for removing target components upon destruction, but this only applies to `AttackTarget`s.  We'll want to expand this feature to also cover `ProjectileTarget`s:\n\n```elixir\ndefmodule Ship.Systems.Destruction do\n  ...\n  alias Ship.Components.ProjectileTarget\n  ...\n  defp untarget(target) do\n    for ship <- AttackTarget.search(target) do\n      AttackTarget.remove(ship)\n      SeekingTarget.add(ship)\n    end\n\n    for projectile <- ProjectileTarget.search(target) do\n      ProjectileTarget.remove(projectile)\n    end\n  end\nend\n```\n\nOur final task is to render these projectiles in the LiveView.  Let's start by adding a new assign:\n\n```elixir\ndefmodule Ship.GameLive do\n  ...\n  alias Ship.Components.IsProjectile\n  ...\n  defp assign_loading_state(socket) do\n    assign(socket,\n      ...\n      projectiles: []\n    )\n  end\n\n  def handle_info(:first_load, socket) do\n    ...\n    socket =\n      socket\n      |> assign_player_ship()\n      |> assign_other_ships()\n      |> assign_projectiles()\n      |> assign_offsets()\n      |> assign(loading: false)\n    ...\n  end\n\n  def handle_info(:refresh, socket) do\n    socket =\n      socket\n      |> assign_player_ship()\n      |> assign_other_ships()\n      |> assign_projectiles()\n      |> assign_offsets()\n    ...\n  end\n  ...\n  defp assign_projectiles(socket) do\n    projectiles =\n      for projectile <- IsProjectile.get_all() do\n        x = XPosition.get(projectile)\n        y = YPosition.get(projectile)\n        image = ImageFile.get(projectile)\n        {projectile, x, y, image}\n      end\n\n    assign(socket, projectiles: projectiles)\n  end\n  ...\nend\n```\n\nThen we'll update the render to include the projectiles:\n\n```elixir\ndefmodule Ship.GameLive do\n  def render(assigns) do\n    ~H\"\"\"\n    ...\n    <%= for {_entity, x, y, image_file} <- @projectiles do %>\n       \n    <% end %>\n    <%= for {_entity, x, y, image_file} <- @other_ships do %>\n    ...\n    \"\"\"\n  end\nend\n```\n\nLastly - `cannonball.svg` - we will make this file from scratch!\n\n    $ touch priv/static/images/cannonball.svg\n\n```html\n \n   \n \n```\n\nThe `width` and `height` values will be overriden by our LiveView render's `width=\"1\" height=\"1\"`, but they still play an important role - because the circle's parameters will be measured relative to these - so we'll set them to `100` for simplicity.  `cx` and `cy` represent the coordinates for the center of the circle, which should be one-half the `width` and `height`.  The size of the circle will be set with `r` (radius) and `stroke-width` (the border around the circle) - we can calculate `diameter = 2 * (r + stroke_width) = 60`.  This diameter is also relative, so when our `cannonball.svg` is scaled down to `1 x 1`, the visible circle will be `0.6 x 0.6`","ref":"web_frontend_liveview.html#projectile-animations"},{"type":"extras","title":"Limiting Ship Movement - Web Frontend with LiveView","doc":"You might notice that that once the ship hits the edge of the world map it keeps moving and disappears from sight. This happens because we keep updating the ship's position regardless of it being within the limits of the map.\n\nTo solve this, let's go to the `Driver` system and limit the position range for the player ship:\n\n```elixir\ndefmodule Ship.Systems.Driver do\n  ...\n  def run do\n    for {entity, x_velocity} <- XVelocity.get_all() do\n      ...\n      new_x_position = calculate_new_position(x_position, x_velocity)\n      ...\n    end\n\n    for {entity, y_velocity} <- YVelocity.get_all() do\n      ...\n      new_y_position = calculate_new_position(y_position, y_velocity)\n      ...\n    end\n    ...\n  end\n\n  # Do not let player ship move past the map limit\n  defp calculate_new_position(current_position, velocity) do\n    new_position = current_position + velocity\n    new_position = Enum.min([new_position, 99])\n\n    Enum.max([new_position, 0])\n  end\nend\n```\n\nThis will limit the position for the ship on both X and Y axis to be between 0 and 99 (the size of the 100x100 world map).","ref":"web_frontend_liveview.html#limiting-ship-movement"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}